@@ -40,12 +40,7 @@ import six
 
 import pyghmi.constants as const
 import pyghmi.exceptions as exc
-import pyghmi.ipmi.private.constants as ipmiconst
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
 
 TYPE_UNKNOWN = 0
 TYPE_SENSOR = 1
@@ -288,15 +283,15 @@ class SDREntry(object):
     external code to pay attention to this class.
     """
 
-    def __init__(self, entrybytes, ipmicmd, reportunsupported=False,
+    def __init__(self, entrybytes, event_consts, reportunsupported=False,
                  mfg_id=0, prod_id=0):
         self.mfg_id = mfg_id
         self.prod_id = prod_id
+        self.event_consts = event_consts
         # ignore record id for now, we only care about the sensor number for
         # moment
         self.readable = True
         self.reportunsupported = reportunsupported
-        self.ipmicmd = ipmicmd
         if entrybytes[2] != 0x51:
             # only recognize '1.5', the only version defined at time of writing
             raise NotImplementedError
@@ -377,9 +372,10 @@ class SDREntry(object):
         # event only, compact and full are very similar
         # this function handles the common aspects of compact and full
         # offsets from spec, minus 6
+        self.sensor_owner = entry[0]
         self.sensor_lun = entry[1] & 0x03
         self.sensor_number = entry[2]
-        self.entity = ipmiconst.entity_ids.get(
+        self.entity = self.event_consts.entity_ids.get(
             entry[3], 'Unknown entity {0}'.format(entry[3]))
         if self.rectype == 3:
             self.sensor_type_number = entry[5]
@@ -388,7 +384,7 @@ class SDREntry(object):
             self.sensor_type_number = entry[7]
             self.reading_type = entry[8]  # table 42-1
         try:
-            self.sensor_type = ipmiconst.sensor_type_codes[
+            self.sensor_type = self.event_consts.sensor_type_codes[
                 self.sensor_type_number]
         except KeyError:
             self.sensor_type = "UNKNOWN type " + str(self.sensor_type_number)
@@ -420,6 +416,8 @@ class SDREntry(object):
             # or hyphen, so go with space
             self.unit_mod = " "
         self.percent = ''
+        # two bytes of assertion event mask / lower threshold reading mask
+        self.assertion_reading_mask = entry[9:11]
         if entry[15] & 1 == 1:
             self.percent = '% '
         if self.sensor_type_number == 0xb:
@@ -454,13 +452,13 @@ class SDREntry(object):
             self.decode_formula(entry[19:25])
 
     def _decode_state(self, state):
-        mapping = ipmiconst.generic_type_offsets
+        mapping = self.event_consts.generic_type_offsets
         try:
             if self.reading_type in mapping:
                 desc = mapping[self.reading_type][state]['desc']
                 health = mapping[self.reading_type][state]['severity']
             elif self.reading_type == 0x6f:
-                mapping = ipmiconst.sensor_type_offsets
+                mapping = self.event_consts.sensor_type_offsets
                 desc = mapping[self.sensor_type_number][state]['desc']
                 health = mapping[self.sensor_type_number][state]['severity']
             elif self.reading_type >= 0x70 and self.reading_type <= 0x7f:
@@ -477,7 +475,7 @@ class SDREntry(object):
             health = const.Health.Warning
         return desc, health
 
-    def decode_sensor_reading(self, reading):
+    def decode_sensor_reading(self, ipmicmd, reading):
         numeric = None
         output = {
             'name': self.sensor_name,
@@ -497,8 +495,8 @@ class SDREntry(object):
         if numeric is not None:
             lowerbound = numeric - (0.5 + (self.tolerance / 2.0))
             upperbound = numeric + (0.5 + (self.tolerance / 2.0))
-            lowerbound = self.decode_value(lowerbound)
-            upperbound = self.decode_value(upperbound)
+            lowerbound = self.decode_value(ipmicmd, lowerbound)
+            upperbound = self.decode_value(ipmicmd, upperbound)
             output['value'] = (lowerbound + upperbound) / 2.0
             output['imprecision'] = output['value'] - lowerbound
             discrete = False
@@ -514,53 +512,55 @@ class SDREntry(object):
         output['health'] = const.Health.Ok
         if discrete:
             for state in range(8):
-                if reading[2] & (0b1 << state):
+                if reading[2] & (0b1 << state) & \
+                        self.assertion_reading_mask[0]:
                     statedesc, health = self._decode_state(state)
                     output['health'] |= health
                     output['states'].append(statedesc)
                     output['state_ids'].append(self.assert_trap_value(state))
             if len(reading) > 3:
                 for state in range(7):
-                    if reading[3] & (0b1 << state):
+                    if reading[3] & (0b1 << state) & \
+                            self.assertion_reading_mask[0]:
                         statedesc, health = self._decode_state(state + 8)
                         output['health'] |= health
                         output['states'].append(statedesc)
                         output['state_ids'].append(
                             self.assert_trap_value(state + 8))
         else:
-            if reading[2] & 0b1:
+            if reading[2] & 0b1 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Warning
                 output['states'].append(lower + " non-critical threshold")
                 output['state_ids'].append(self.assert_trap_value(1))
-            if reading[2] & 0b10:
+            if reading[2] & 0b10 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Critical
                 output['states'].append(lower + " critical threshold")
                 output['state_ids'].append(self.assert_trap_value(2))
-            if reading[2] & 0b100:
+            if reading[2] & 0b100 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Failed
                 output['states'].append(lower + " non-recoverable threshold")
                 output['state_ids'].append(self.assert_trap_value(3))
-            if reading[2] & 0b1000:
+            if reading[2] & 0b1000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Warning
                 output['states'].append(upper + " non-critical threshold")
                 output['state_ids'].append(self.assert_trap_value(4))
-            if reading[2] & 0b10000:
+            if reading[2] & 0b10000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Critical
                 output['states'].append(upper + " critical threshold")
                 output['state_ids'].append(self.assert_trap_value(5))
-            if reading[2] & 0b100000:
+            if reading[2] & 0b100000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Failed
                 output['states'].append(upper + " non-recoverable threshold")
                 output['state_ids'].append(self.assert_trap_value(6))
         return SensorReading(output, self.unit_suffix)
 
-    def _set_tmp_formula(self, value):
-        rsp = self.ipmicmd.raw_command(netfn=4, command=0x23,
-                                       data=(self.sensor_number, value))
+    def _set_tmp_formula(self, ipmicmd, value):
+        rsp = ipmicmd.raw_command(netfn=4, command=0x23,
+                                  data=(self.sensor_number, value))
         # skip next reading field, not used in on-demand situation
         self.decode_formula(rsp['data'][1:])
 
-    def decode_value(self, value):
+    def decode_value(self, ipmicmd, value):
         # Take the input value and return meaningful value
         linearization = self.linearization
         if linearization > 11:  # direct calling code to get factors
@@ -570,7 +570,7 @@ class SDREntry(object):
             # fashion.  However for now opt for retrieving rows as needed
             # rather than tracking all that information for a relatively
             # rare behavior
-            self._set_tmp_formula(value)
+            self._set_tmp_formula(ipmicmd, value)
             linearization = 0
         # time to compute the pre-linearization value.
         decoded = float((value * self.m + self.b)
@@ -736,15 +736,17 @@ class SDR(object):
         cachefilename = None
         self.broken_sensor_ids = {}
         if self.cachedir:
-            cachefilename = 'sdrcache.{0}.{1}.{2}.{3}.{4}.{5}'.format(
+            cachefilename = 'sdrcache-2.{0}.{1}.{2}.{3}.{4}.{5}'.format(
                 self.mfg_id, self.prod_id, self.device_id, self.fw_major,
                 self.fw_minor, modtime)
             cachefilename = os.path.join(self.cachedir, cachefilename)
         if cachefilename and os.path.isfile(cachefilename):
             with open(cachefilename, 'rb') as cfile:
-                csdrs = pickle.load(cfile)
-                for sdrdata in csdrs:
-                    self.add_sdr(sdrdata)
+                csdrlen = cfile.read(2)
+                while csdrlen:
+                    csdrlen = struct.unpack('!H', csdrlen)[0]
+                    self.add_sdr(cfile.read(csdrlen))
+                    csdrlen = cfile.read(2)
                 for sid in self.broken_sensor_ids:
                     try:
                         del self.sensors[sid]
@@ -774,7 +776,7 @@ class SDR(object):
                     if size == 0xff:  # get just 5 to get header to know length
                         size = 5
                     elif size > 5:
-                        size /= 2
+                        size //= 2
                         # push things over such that it's less
                         # likely to be just 1 short of a read
                         # and incur a whole new request
@@ -803,7 +805,7 @@ class SDR(object):
                     size = currlen - offset
             self.add_sdr(sdrdata)
             if sdrraw is not None:
-                sdrraw.append(sdrdata)
+                sdrraw.append(bytes(sdrdata))
             offset = 0
             if size != 0xff:
                 size = 5
@@ -824,7 +826,9 @@ class SDR(object):
             suffix = ''.join(
                 random.choice(string.ascii_lowercase) for _ in range(12))
             with open(cachefilename + '.' + suffix, 'wb') as cfile:
-                pickle.dump(sdrraw, cfile)
+                for csdr in sdrraw:
+                    cfile.write(struct.pack('!H', len(csdr)))
+                    cfile.write(csdr)
             os.rename(cachefilename + '.' + suffix, cachefilename)
 
     def get_sensor_numbers(self):
@@ -832,11 +836,15 @@ class SDR(object):
             if self.sensors[number].readable:
                 yield number
 
+    def make_sdr_entry(self, sdrbytes):
+        return SDREntry(sdrbytes, self.ipmicmd.get_event_constants(),
+                        False, self.mfg_id, self.prod_id)
+
     def add_sdr(self, sdrbytes):
-        newent = SDREntry(sdrbytes, self.ipmicmd, False, self.mfg_id,
-                          self.prod_id)
+        newent = self.make_sdr_entry(sdrbytes)
         if newent.sdrtype == TYPE_SENSOR:
-            id = '{0}.{1}'.format(newent.sensor_number, newent.sensor_lun)
+            id = '{0}.{1}.{2}'.format(
+                newent.sensor_owner, newent.sensor_number, newent.sensor_lun)
             if id in self.sensors:
                 self.broken_sensor_ids[id] = True
                 return
